import { useCallback, useEffect, useState } from 'react'
import {
  createVirtualBackground,
  LocalMediaStream,
  StreamKind,
  StreamSource,
  VirtualBackground,
  VirtualBackgroundType,
} from '@zocks-communications/client-sdk-js'
import { useZocksDispatch, useZocksStore } from '../../app/hooks'
import {
  clearCurrentVirtualBackground,
  selectCurrentVirtualBackground,
  setCurrentVirtualBackground,
} from '../virtualBackground/virtualBackgroundSlice'
import { useMediaDevice } from '../mediaDevice'
import { setSelectedMediaDevice, setVideoIsLoading } from '../mediaDevice/mediaDeviceSlice'

export const useLocalMedia = (videoEnabledByDefault?: boolean, audioEnabledByDefault?: boolean) => {
  const dispatch = useZocksDispatch()
  const { currentAudioDeviceId, currentVideoDeviceId } = useMediaDevice()
  const [localAudioStream, setLocalAudioStream] = useState<LocalMediaStream>()
  const [localVideoStream, setLocalVideoStream] = useState<LocalMediaStream>()

  const setSelectedMediaDeviceId = useCallback(
    (stream: LocalMediaStream, kind: MediaDeviceKind) => {
      const deviceId = stream.mediaStreamTrack?.getSettings().deviceId
      if (deviceId) {
        dispatch(setSelectedMediaDevice({ kind, deviceId }))
      }
    },
    [dispatch]
  )

  const startLocalAudioStream = useCallback(
    (deviceId?: string) => {
      LocalMediaStream.create({ audio: { deviceId }, video: false }).then((audioStream) => {
        setLocalAudioStream(audioStream)
        setSelectedMediaDeviceId(audioStream, 'audioinput')
      })
    },
    [setSelectedMediaDeviceId]
  )

  const startLocalVideoStream = useCallback(
    (deviceId?: string) => {
      dispatch(setVideoIsLoading(true))
      LocalMediaStream.create({ audio: false, video: { deviceId } })
        .then((videoStream) => {
          setLocalVideoStream(videoStream)
          setSelectedMediaDeviceId(videoStream, 'videoinput')
        })
        .finally(() => dispatch(setVideoIsLoading(false)))
    },
    [dispatch, setSelectedMediaDeviceId]
  )

  const toggleLocalAudioStream = () => {
    if (localAudioStream) {
      localAudioStream.stopStream()
      setLocalAudioStream(undefined)
    } else {
      startLocalAudioStream(currentAudioDeviceId || undefined)
    }
  }

  const toggleLocalVideoStream = () => {
    if (localVideoStream) {
      localVideoStream.stopStream()
      setLocalVideoStream(undefined)
    } else {
      startLocalVideoStream(currentVideoDeviceId || undefined)
    }
  }

  const selectAudioDevice = (audioDevice: MediaDeviceInfo) => {
    if (localAudioStream) {
      if (localAudioStream.mediaStreamTrack?.getSettings().deviceId === audioDevice.deviceId) {
        return
      }
      localAudioStream.stopStream()
      startLocalAudioStream(audioDevice.deviceId)
    }
  }

  const selectVideoDevice = (videoDevice: MediaDeviceInfo) => {
    if (localVideoStream) {
      if (localVideoStream.mediaStreamTrack?.getSettings().deviceId === videoDevice.deviceId) {
        return
      }
      localVideoStream.stopStream()
      setLocalVideoStream(undefined)
      startLocalVideoStream(videoDevice.deviceId)
    }
  }

  useEffect(() => {
    if (videoEnabledByDefault) {
      startLocalVideoStream()
    }
  }, [videoEnabledByDefault, startLocalVideoStream])

  useEffect(() => {
    if (audioEnabledByDefault) {
      startLocalAudioStream()
    }
  }, [audioEnabledByDefault, startLocalAudioStream])

  return {
    currentAudioDeviceId,
    currentVideoDeviceId,
    localAudioStream,
    localVideoStream,
    toggleLocalAudioStream,
    toggleLocalVideoStream,
    selectAudioDevice,
    selectVideoDevice,
  }
}

export const useLocalVirtualBackgrounds = () => {
  const dispatch = useZocksDispatch()
  const currentVirtualBackground = useZocksStore(selectCurrentVirtualBackground)
  const availableVirtualBackgrounds = Object.values(VirtualBackgroundType)

  const [virtualBackground, setVirtualBackground] = useState<VirtualBackground>()
  const [virtualBackgroundStream, setVirtualBackgroundStream] = useState<LocalMediaStream>()

  const changeVirtualBackground = async (virtualBackgroundType: VirtualBackgroundType | null, deviceId?: string) => {
    try {
      dispatch(setVideoIsLoading(true))
      if (virtualBackgroundType) {
        await createNewVirtualBackground(virtualBackgroundType, deviceId)
        dispatch(setCurrentVirtualBackground(virtualBackgroundType))
      } else {
        await stopVirtualBackground()
        dispatch(clearCurrentVirtualBackground())
      }
    } finally {
      dispatch(setVideoIsLoading(false))
    }
  }

  const createNewVirtualBackground = async (virtualBackgroundType: VirtualBackgroundType, deviceId?: string) => {
    await virtualBackground?.stop()
    const sourceVideoStream = await LocalMediaStream.create({
      audio: false,
      video: { deviceId, width: { ideal: 1280 }, height: { ideal: 720 } },
    })
    const newVirtualBackground = createVirtualBackground(virtualBackgroundType, sourceVideoStream.mediaStream!)
    await newVirtualBackground.start()
    const targetVideoStream = LocalMediaStream.createFromMediaStreamTrack(
      newVirtualBackground.outputStream,
      newVirtualBackground.outputStreamTrack,
      StreamKind.VIDEO,
      StreamSource.CAMERA
    )
    setVirtualBackgroundStream(targetVideoStream)
    setVirtualBackground(newVirtualBackground)
  }

  const stopLocalVirtualBackground = async () => {
    if (!virtualBackground) {
      return
    }
    await stopVirtualBackground()
  }

  const stopVirtualBackground = async () => {
    await virtualBackground?.stop()
    setVirtualBackgroundStream(undefined)
    setVirtualBackground(undefined)
  }

  return {
    availableVirtualBackgrounds,
    currentVirtualBackground,
    virtualBackgroundStream,
    changeVirtualBackground,
    stopLocalVirtualBackground,
  }
}
